package EFS::CLI::DBServer;

use strict;
use warnings;
use English;

use base q{EFS::CLI};
use base q{EFS::DBI};

use Params::Validate qw( :all );

sub define {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        type      => { type => SCALAR },
        efsserver => { type => SCALAR },
    });

    my $efsserver_rs = $class->get_resultset(q{EFSServer});
    my $efsserver =
        $efsserver_rs->retrieve_bynames_assert( name => $args{efsserver} );

    if ( $args{type} ne q{primary} and $args{type} ne q{secondary} ) {
        die qq{Invalid argument: -type must be either primary or secondary\n};
    }

    # Do we allow the primary to be redefined?  I tend to think that moving
    # the primary should be impossible.  Changing the primary/secondary roles
    # should be allowed, which basically just changes the direction of
    # replication.  Changing the secondary would mean undefining the secondary
    # (shut down replication), defining a new one (setup replication again).

    return 1;

}

sub undef {

    # Allow the secondary to be undefined.  This would break the replication
    # relationship, until a new secondary is defined.
    die(
        qq{Dbserver may not be undefined\n},
        qq{There should always be two: a primary and secondary\n},
    );

    return;

}

sub swaptypes {

    my $class = shift;

    my $resultset = $class->get_resultset;

    my ($primary)   = $resultset->search( { type => q{primary} } );
    my ($secondary) = $resultset->search( { type => q{secondary} } );

    if ( not $primary or not $secondary ) {
        die(
            qq{Both the primary and secondary Dbservers must be defined\n},
            qq{in order to swap their types\n},
        );
    }

    $primary->set_column( type => q{temporary} );
    $primary->update;

    $secondary->set_column( type => q{primary} );
    $secondary->update;

    $primary->set_column( type => q{secondary} );
    $primary->update;

    # This command should also change the direction of replication

    STDOUT->print(
        qq{The primary and secondary Dbservers have swapped types\n},
    );

    return 1;

}

sub history {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        %EFS::CLI::History::validation_specs,
    });

    return EFS::CLI::History->history(%args);

}

sub initdb {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        type     => { type => SCALAR },
        force    => { type => SCALAR, optional => 1 },
        name     => { type => SCALAR, optional => 1 },
        hosttype => { type => SCALAR, optional => 1 },
    });

    my $resultset = $class->get_resultset;
    my ($primary) = $resultset->search( { type => q{primary} } );

    if ( not $primary ) {
        die qq{No primary Dbserver has been defined yet.\n};
    }

    if ( not $primary->efsserver->is_localhost ) {
        die( q{_EFS_REDIRECT_SERVER: } . $primary->efsserver->name . qq{\n} );
    }

    # So....  what exactly do we do to set up a database from scratch
    # anyway?

    return 1;

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
