package EFS::CLI::UserRights;

use strict;
use warnings;
use English;

use base q{EFS::CLI};
use base q{EFS::DBI};
use Params::Validate qw( :all );

sub define {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        metaproj => { type => SCALAR },
        role     => { type => SCALAR },
        username => { type => SCALAR },
    });

    my $metaproj_rs = $class->get_resultset( q{Metaproj} );

    my $metaproj = $metaproj_rs->retrieve_bynames_assert( name => $args{metaproj} );

    if ( not $args{role} ~~ [qw( dev qa prod auto admin )] ) {
        die(
            qq{Invalid role '$args{role}'\n},
            qq{Must be one of: dev, qa, prod, auto or admin\n},
        );
    }

    my $username = EFS::Username->new({ name => $args{username} });

    my $resultset = $class->get_resultset;

    my ($userrights) = $resultset->search({
        metaproj => $metaproj->id,
        role     => $args{role},
        username => $username->name,
    });

    if ( $userrights ) {

        STDOUT->print(
            qq{Username $username already has $args{role} rights to $metaproj\n},
        );

    } else {

        $userrights = $resultset->create({
            metaproj => $metaproj->id,
            role     => $args{role},
            username => $username->name,
        });

        STDOUT->print( qq{Granted $username $args{role} rights to $metaproj\n} );

    }

    return $userrights;

}

sub undef {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        metaproj => { type => SCALAR },
        role     => { type => SCALAR },
        username => { type => SCALAR },
    });

    my $resultset = $class->get_resultset;

    my $userrights = $resultset->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        role     => $args{role},
        username => $args{username},
    );

    $userrights->delete;

    STDOUT->print( qq{Removed $args{username} $args{role} rights to $args{metaproj}\n} );

    return 1;

}

sub display {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        metaproj => { type     => SCALAR,
                      optional => 1 },
        role     => { type     => SCALAR,
                      depends  => [qw( metaproj )],
                      optional => 1 },
        username => { type     => SCALAR,
                      depends  => [qw( role )],
                      optional => 1 },
    });

    my $metaproj_rs = $class->get_resultset( q{Metaproj} );

    my $metaproj = $metaproj_rs->retrieve_bynames_assert( name => $args{metaproj} );

    my @userrights = ();

    my $resultset = $class->get_resultset;

    if ( $args{username} ) {
        $userrights[0] = $resultset->retrieve_bynames_assert(
            metaproj => $args{metaproj},
            role     => $args{role},
            username => $args{username},
        );
    } elsif ( $args{role} ) {
        @userrights = $resultset->search({
            metaproj => $metaproj->id,
            role     => $args{role},
        });
    } elsif ( $args{metaproj} ) {
        @userrights = $resultset->search({
            metaproj => $metaproj->id,
        });
    } else {
        @userrights = $resultset->all;
    }

    if ( @userrights ) {
        $class->display_objects( objects => \@userrights );
    } else {
        STDOUT->print( qq{No userrights match the specified criteria.\n} );
    }

    return 1;

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
