package EFS::DBI::ResultSet::AFSVolume;

use strict;
use warnings;
use English;

use base q{EFS::DBI::ResultSet};

use Readonly;
use Params::Validate qw( :all );

Readonly my $AFS_VOLNAME_MAXLEN => 22;

sub validate_data {

    my $class = shift;
    my %data  = @_;

    if ( exists $data{type} ) {
        $data{type} ~~ [qw( root dev dist home )] || die(
            qq{Invalid afsvolume type '$data{type}'\n},
            qq{Must be one of root, dev, dist or home\n},
        );
    }

    if ( exists $data{afsvolume} ) {

        length $data{afsvolume} <= $AFS_VOLNAME_MAXLEN || die(
            qq{Invalid afsvolume name '$data{afsvolume}'\n},
            qq{Must be no more than $AFS_VOLNAME_MAXLEN\n},
        );

        $data{afsvolume} =~ m/\A [-\w\.]+ \z/msx || die(
            qq{Invalid afsvolume name '$data{afsvolume}'\n},
            q{Must consist solely of alphanumeric characters, },
            qq{underscores, dashes and/or dots\n}
        );

    }

    return;
}

sub retrieve {

    my $self = shift;

    my (%args) = validate( @_, {
        type   => { type => SCALAR },
        name   => { type => SCALAR },
    });

    my ($afsvolume) = $self->search({ name => $args{name} });
    
    if ( not $afsvolume ) {
        $afsvolume = $self->create({
            type => $args{type},
            name => $args{name},
        });
    }

    return $afsvolume;

}

sub register {

    my $self = shift;

    my (%args) = validate( @_, {
        type   => { type => SCALAR },
        parent => { type => OBJECT },
    });

    my $type   = $args{type};
    my $parent = $args{parent};

    my $metaproj = $parent->isa( q{EFS::DBI::Result::Metaproj} ) ?
        $parent->name : $parent->metaproj->name;

    my $afsvolume = $self->create({
        type       => $type,
        name       => qq{tempname.$PID},
        parenttype => $parent->class2type,
        parentid   => $parent->id,
    });

    my $id     = $afsvolume->id;
    my $prefix = qq{efs.$type.$metaproj};

    while ( length qq{$prefix.$id} > $AFS_VOLNAME_MAXLEN ) {
        $prefix =~ s{.$}{}ms;
    }

    $afsvolume->set_column( name => qq{$prefix.$id} );
    $afsvolume->update;

    return $afsvolume;

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
