package EFS::Diststate::ReleaseAlias;

use strict;
use warnings;
use English;

use Moose;
extends qw(EFS::Diststate);

use Params::Validate qw( :all );
use List::MoreUtils qw( uniq all );
use Set::Scalar;
use EFS::Colors;
use Readonly;

Readonly my $SCREEN_WIDTH => 1024;

my $EMPTY_STRING = q{};

has target_by_cell => (
    is => 'rw',
    default => sub { {} },
);

has target => (
    is => 'rw',
);

has releasealias => (
    is => 'rw',
);

sub BUILD {

    my ($self,@args) = @_;

    my (%args) = validate( @args, {
        releasealias => {
            type => OBJECT,
            isa  => q{EFS::DBI::Result::ReleaseAlias},
        },
    });

    my $releasealias = $self->releasealias( $args{releasealias} );

    my $db_target = $self->target( $releasealias->target->name );

    my @cells = ();

    my $cell_rs = EFS::DBI->get_resultset( q{Cell} );

    if ( $releasealias->has_prod_stages ) {
        @cells  = $cell_rs->all;
    } else {
        @cells  = (
            $cell_rs->search({ celltype => q{dev} }),
            $cell_rs->search({ celltype => q{test} }),
        );
    }

    for my $cell ( @cells ) {

        # Slave cells have no distinct diststate, and they simply
        # inherit the diststate of their parent prod cell.
        if ( $cell->celltype eq q{prod} &&
                 $cell->parentcell->celltype eq q{prod} ) {
            $cell = $cell->parentcell;
        }

        my $target = $releasealias->target_for_cell($cell);

        if ( $target ) {
            $self->target_by_cell()->{$cell} = $target;
        } else {
            $self->target_by_cell()->{$cell} = $EMPTY_STRING;
        }

    }

    return 1;

}

sub related_item { return shift->get_releasealias }

sub subitem_diststates { return }   # releasealiases have no subitems

sub needs_update {

    my $self = shift;

    my (%args) = validate( @_, {
        cells => { type => ARRAYREF, optional => 1 },
    });

    if ( $self->cells_needing_update(%args) ) {
        return 1;
    } else {
        return;
    }

}

sub releasealiases_needing_update {

    my $self = shift;

    my (%args) = validate( @_, {
        cells => { type => ARRAYREF, optional => 1 },
    });

    if ( $self->cells_needing_update(%args) ) {
        return $self->get_releasealias;
    } else {
        return;
    }

}

sub cells_needing_update {

    my $self  = shift;

    my (%args) = validate( @_, {
        cells => { type => ARRAYREF, optional => 1 },
    });

    return $self->_gather_cells_matching(
        filter => sub {
            # we want the cells where the symlink points somewhere other than
            # the expected target
            $self->target_by_cell->{ $_[0] }
                ne $self->target;
        },
        %args
    );

}

sub has_been_updated {

    my $self  = shift;

    my (%args) = validate( @_, {
        cells => { type => ARRAYREF, optional => 1 },
    });

    # If cells argument not given, we use all known cells
    my @wanted_cells = ref($args{cells}) ? @{ $args{cells} } : ();
    if ( not @wanted_cells ) {
        @wanted_cells = keys %{ $self->target_by_cell };
    }

    # We need to be sure *all* cells in %args have been updated
    my $wanted_cellset =
        Set::Scalar->new( map { ref $_ ? $_->name : $_ } @wanted_cells );
    my $updated_cellset =
        Set::Scalar->new( map { $_->name } $self->cells_updated(%args) );

    if ( $wanted_cellset == $updated_cellset ) {
        return 1;
    } else {
        return;
    }

}

sub releasealiases_updated {

    my $self  = shift;

    my (%args) = validate( @_, {
        cells => { type => ARRAYREF, optional => 1 },
    });

    if ( $self->has_been_updated(%args) ) {
        return $self;
    } else {
        return;
    }

}

sub cells_updated {

    my $self  = shift;

    my (%args) = validate( @_, {
        cells => { type => ARRAYREF, optional => 1 },
    });

    return $self->_gather_cells_matching(
        filter => sub {
            # we want the cells where there is a symlink (might be pointing at
            # the wrong target)
            $self->target_by_cell->{ $_[0] }
        },
        %args
    );

}

# For a releasealias, the concept of partial update degenerates to just
# the list of cells updated, so we make cells_partially_updated an
# alias for cells_updated

{
    no strict q{refs}; ## no critic (ProhibitNoStrict)
    *{cells_partially_updated} = \&cells_updated;
}

# Factors out identical code from needs_update and has_been_updated
sub _gather_cells_matching {

    my $self  = shift;

    my (%args) = validate( @_, {
        cells  => { type => ARRAYREF, optional => 1 },
        filter => { type => CODEREF },
    });

    my %interested_in = map { $_ => $_ } @{ $args{cells} };

    my @results = ();

    my $cell_rs = EFS::DBI->get_resultset( q{Cell} );

    foreach my $cell_name ( keys %{ $self->target_by_cell } ) {
        if ( @{ $args{cells} } ) { # if option not given, check all cells
            next unless $interested_in{$cell_name};
        }
        if ( $args{filter}->($cell_name) ) {
            push @results,
                $cell_rs->retrieve_bynames_assert( name => $cell_name );
        }
    }

    return @results;

}

# This is a hack.  Release aliases don't have checkpoints, but the
# _text_status method checks that there is a checkpoint for other
# types of items.  So we return a 1 just so that it checks true.

sub get_checktime {
    return 1;
}

sub display_text_table {

    my $self = shift;

    my (%args) = validate( @_, {
        cells                 => { type => ARRAYREF },
        verbose               => { type => BOOLEAN | UNDEF, default => 0 },
        respect_cell_ordering => { type => BOOLEAN | UNDEF, default => 0 },
    });

    # We're going to modify the cell list, so make it a copy
    $args{cells} = [ @{$args{cells}} ];

    my $output_needed = $args{verbose};

    my $wanted_target = $self->target;
    my @output = (
        q{Releasealias target: } . color_good($wanted_target) . qq{\n}
    );

    my %cells_by_type;
    foreach my $cell ( @{ $args{cells} } ) {
        push @{ $cells_by_type{$cell->celltype} }, $cell;
    }

  CELLTYPE:
    foreach my $cell_type ( sort keys %cells_by_type ) {

        my @output_for_celltype;

        my $table_needed = $args{verbose};

        my @cells = @{ $cells_by_type{$cell_type} };

        if ( not $args{respect_cell_ordering} ) {
            @cells = EFS::DBI->sort_objects(@cells);
        }

        push @output_for_celltype, ( $self->_celltype_marker($cell_type) );

        my @headers      = qw(Cell Target);
        my @table_format = map { q{l} } @headers;

        my $table = Text::FormatTable->new(
            join( q{  }, @table_format )
        );

        $table->head(@headers);
        $table->rule(q{=});

        my $uptodate_cell_count = 0;

      CELL:
        foreach my $cell (@cells) {
            my $target = $self->target_by_cell->{$cell};
            if ( $target eq $wanted_target ) {
                if ( $args{verbose} ) {
                    $table->row( $cell->shortname, color_good($target) );
                }
                else {
                    $uptodate_cell_count++;
                }
            }
            elsif ($target) {
                $table->row( $cell->shortname, color_warn($target) );
                $table_needed++;
            }
            else {
                $table->row( $cell->shortname, color_bad(q{---}) );
                $table_needed++;
            }
        }

        if ($table_needed) {
            $table->rule;
            push @output_for_celltype, ( $table->render($SCREEN_WIDTH) );
            push @output, @output_for_celltype;
            if ($uptodate_cell_count) {
                push @output, $self->_table_counts(
                    up_to_date_count => $uptodate_cell_count,
                    item => q{cell},
                );
            }
            $output_needed++;
        }
        else {
            STDOUT->print(
                $self->_celltype_uptodate_message($cell_type)
            );
            # Remove these cells from %args so they won't be
            # displayed in the cell equivalence legend
            @{$args{cells}} =
                grep { $_->celltype ne $cell_type } @{$args{cells}};
        }
    }

    if ($output_needed) {
        STDOUT->print(@output);
        STDOUT->print(
            $self->_table_legend( verbose => $args{verbose} )
        );
    }

    return;

}

sub _table_legend {

    my $self = shift;

    my (%args) = validate( @_, {
        verbose => { type => BOOLEAN | UNDEF, default => 0 },
    });

    my $table = Text::FormatTable->new( q{r l} );

    if ( $args{verbose} ) {
        $table->row(
            color_good( $self->get_db_target ) . q{:},
            q{Releasealias is disted to cell and up to date}
        );
    }

    $table->row(
        color_warn( q{NNN} ) . q{:},
        q{Releasealias is disted but target is not up to date}
    );
    $table->row(
        color_bad( q{---} ) . q{:},
        q{Releasealias is missing from cell}
    );

    return qq{Legend:\n} . $table->render($SCREEN_WIDTH);

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
