package EFS::ForkManager;

use strict;
use warnings;
use English;

use Moose;
use Moose::Util::TypeConstraints;
use Parallel::ForkManager;
use Params::Validate qw( :all );
use Regexp::Common;
use File::Temp;
use Try::Tiny;

has q{action} => (
    is       => q{ro},
    isa      => q{CodeRef},
    required => 1,
);

has q{list} => (
    is       => q{ro},
    isa      => q{ArrayRef},
    required => 1,
);

has q{parallel} => (
    is       => q{rw},
    isa      => q{Int},
    default  => 8,
);

has q{trapoutput} => (
    is       => q{rw},
    isa      => q{Bool},
    default  => 1,
);

has q{showoutput} => (
    is      => q{rw},
    isa     => enum( [ qw(finish run) ] ),
    default => q{finish},
);

has q{forkmanager} => (
    is         => q{ro},
    isa        => q{Parallel::ForkManager},
    lazy_build => 1,
    init_arg   => undef,
);

has q{retrycount} => (
    is         => q{rw},
    isa        => q{Int},
    default    => 10,
);

has q{retrysleep} => (
    is      => q{rw},
    isa     => q{Int},
    default => 60,
);

has q{failed} => (
    traits   => [q{Array}],
    is       => q{ro},
    isa      => q{ArrayRef[Item]},
    init_arg => undef,
    default  => sub { [] },
    handles  => {
        get_failed => q{elements},
        set_failed => q{push}
    },
);

has q{succeeded} => (
    traits   => [q{Array}],
    is       => q{ro},
    isa      => q{ArrayRef[Item]},
    init_arg => undef,
    default  => sub { [] },
    handles  => {
        get_succeeded => q{elements},
        set_succeeded => q{push}
    },
);

has q{_tempfile} => (
    is      => q{ro},
    isa     => q{HashRef[FileHandle]},
    default => sub { {} },
);

has q{_index} => (
    is         => q{ro},
    isa        => q{HashRef[Int]},
    default    => sub { {} },
);

has q{_output} => (
    is      => q{ro},
    isa     => q{ArrayRef[Str]},
    default => sub { [] },
);



sub BUILD {

    my $self = shift;

    if ( $self->parallel < 0 ) {
        die(
            q{Invalid parallel value '} . $self->parallel . q{'. },
            qq{Must be a non-negative integer\n}
        );
    }

    if ($PERLDB) {
        # If we're debugging, forking is painful
        $self->parallel( 0 );
        $self->trapoutput( 0 );
    }
    elsif ( $ENV{EFS_TEST_SUITE} ) {
        # We need to trap ALL output when running the test suite,
        # so that we can get deterministic output for comparison
        # purposes.  We also need to see it sorted, so show it
        # during the run method.
        $self->trapoutput( 1 );
        $self->showoutput( q{run} );
    }
    elsif ( $self->parallel == 1 ) {
        # If we're single threading, then there's no point in
        # trapping the output.  Let it flow through.
        $self->trapoutput( 1 );
    }

    my $max_parallel = $self->get_max_parallel;

    if ( $self->parallel > $max_parallel ) {
        $self->parallel( $max_parallel );
    }

    return;
}

sub _build_forkmanager {

    my $self = shift;

    STDOUT->autoflush(1);
    STDERR->autoflush(1);

    my $forkmanager = Parallel::ForkManager->new( $self->parallel );

    # XXX: Workaround until we get the patched P::FM on CPAN
    # This is really an optional feature anyway.
    if ( $forkmanager->can(q{set_retrycount}) ) {
        $forkmanager->set_retrycount( $self->retrycount );
        $forkmanager->set_retrysleep( $self->retrysleep );
    }

    $forkmanager->run_on_finish(
        sub {

            my ( $pid, $exit_code, $element ) = @_;

            if ($exit_code) {
                $self->set_failed($element);
            }
            else {
                $self->set_succeeded($element);
            }

            if ( $self->trapoutput ) {

                my $fh = $self->_tempfile->{$pid};
                seek( $fh, 0, 0 );
                my $index = $self->_index->{$pid};

                my $tmp_filename = $fh->filename;

                $self->_output->[$index] = q{};

                while ( defined ( $_ = $fh->getline ) ) {
                    $self->_output->[$index] .= $_;
                }

                $fh->close || die(
                    q{Unable to close temporary file },
                    qq{$tmp_filename: $ERRNO\n},
                );

                unlink ($tmp_filename) || die(
                    q{Unable to unlink temporary file },
                    qq{$tmp_filename: $ERRNO\n},
                );

                if ( $self->showoutput eq q{finish} ) {
                    STDOUT->print( $self->_output->[$index] );
                }

            }

        }
    );

    return $forkmanager;

}

sub get_max_parallel {

    my $class = shift;

    my $ptable = EFS::Proc::ProcessTable->new;

    my $process;

    foreach my $proc ( @{ $ptable->table } ) {
        if ( $proc->pid == $PID ) {
            $process = $proc;
            last;
        }
    }

    if ( not $process ) {
        die qq{Unable to determine process object for $PID\n};
    }

    # When testing this code, we pass in a fake size
    # See t/root/02_forkmanager.t
    my $size = shift || $process->size;

    ##  no critic (MagicNumbers)
    if ( $size > 2_000_000_000 ) {
        return 2;
    }
    elsif ( $size > 1_000_000_000 ) {
        return 4;
    }
    else {
        return 8;
    }

}

sub run {

    my $self  = shift;

    my $forkmanager = $self->forkmanager;

    my @list = @{ $self->list };

    foreach my $index ( 0 .. $#list ) {

        my $element = $list[$index];

        my $tmp_fh = undef;

        if ( $self->trapoutput ) {
            $tmp_fh = File::Temp->new( unlink => 0 );
        }

        my $child = $forkmanager->start($element);

        if ($child) {

            $self->_tempfile->{$child} = $tmp_fh;
            $self->_index->{$child}    = $index;
            next;

        }
        else {

            if ( $self->trapoutput ) {
                STDOUT->fdopen( $tmp_fh->fileno, q{w} ) ||
                    die( qq{Unable to redirect STDOUT: $ERRNO\n} );
                STDERR->fdopen( STDOUT->fileno, q{w} ) ||
                    die( qq{Unable to redirect STDERR: $ERRNO\n} );
            }

            EFS::DBI->inactivate_handles;

            my $return;

            my $error;
            try { $return = $self->action->($element); }
            catch { $error = $_; STDOUT->print($_) };

            $forkmanager->finish( $error ? 1 : ( $return ? 0 : 1 ) );

        }

    }

    $forkmanager->wait_all_children;

    if ( $self->trapoutput && $self->showoutput eq q{run} ) {
        foreach my $index ( 0 .. $#list ) {
            STDOUT->print( $self->_output->[$index] );
        }
    }

    return ( $self->get_failed ? 0 : 1 );

}


no Moose;
__PACKAGE__->meta->make_immutable;
1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
