package EFS::Logfile;

use strict;
use warnings;
use English;

use EFS::FQDN qw(fqdn);

use IO::File;
use Date::Format;
use Errno;
use File::Basename qw(dirname);
use Params::Validate qw( :all );

sub new {

    my $class = shift;
    my $self = {};

    bless $self, $class;

    my $dir =
        join( q{/}, $EFS::logroot,
              time2str( q{%Y/%m/%d}, time, q{GMT} ) );

    EFS::Util->MakeDirHierarchy(
        top    => $EFS::logroot,
        bottom => $dir,
    );

    my $index = 0;

  INIT:
    {
        $self->{path}   = join(q{/},$dir,join(q{.},time,$index));
        $self->{handle} = IO::File->new($self->{path},
                                        O_CREAT|O_EXCL|O_WRONLY) || do {
            if ( $ERRNO == Errno::EEXIST ) {
                $index++;
                redo INIT;
            }
            else {
                die(qq{Unable to open $self->{path}: $ERRNO});
            }
        };
    }

    return $self;

}

sub handle {
    return shift->{handle};
}

sub path {
    return shift->{path};
}

sub archive {

    my $class = shift;

    my (%args) = validate( @_, {
        path    => { type     => SCALAR },
        force   => { type     => SCALAR,
                     optional => 1 },
        verbose => { type     => SCALAR,
                     optional => 1 },
    });

    my $parameter_rs = EFS::DBI->get_resultset( q{Parameter} );

    if ( $parameter_rs->configured( q{disable_logfile_archive} )
             and not $args{force} ) {
        return 1;
    }

    my $path = $args{path};

    my $efshost = fqdn;

    my $config = EFS::Config::Client->new;

    my $cell_rs = EFS::DBI->get_resultset( q{Cell} );

    my $cell = $cell_rs->retrieve_bynames_assert(
        name => $config->get_value( q{client}, q{cell} ),
    );

    my $logs = q{logs-} . $cell->shortname;

    my $logs_root = qq{/efs/dev/efs/$logs/incr/src};

    if ( not -d $logs_root ) {
        if ( $args{verbose} ) {
            warn qq{Project for archiving logs not yet created.\n};
        }
        return;
    }

    my $history_rs = EFS::DBI->get_resultset( q{History} );

    my ($history) = $history_rs->search({
        efshost => $efshost,
        logfile => $path,
    });

    if ( not $history ) {
        if ( $args{verbose} ) {
            STDOUT->print(
                qq{No history entries found for logfile $path\n}
            );
        }
        unlink($path) ||
            die qq{Unable to unlink $path: $ERRNO};
        return 1;
    }

    if ( not defined $history->status ) {
        if ( $args{verbose} ) {
            STDOUT->print(
                qq{Skipping archive of active logfile $path\n},
            );
        }
        return 1;
    }

    my $archive_root = qq{/efs/dev/efs/$logs/incr/src/$efshost};

    my $archive_path = $path;
    $archive_path    =~ s{$EFS::logroot}{$archive_root}msx;
    $archive_path   .= q{.gz};

    EFS::Util->MakeDirHierarchy(
        top    => $archive_root,
        bottom => dirname($archive_path),
    );

    my $gzip_command = qq{gzip -c > $archive_path < $path};

    my @gzip_output = qx{ $gzip_command 2>&1 };

    $CHILD_ERROR and die(
        qq{Unable to run zip command $gzip_command\n},
        @gzip_output
    );

    $history->set_column( logfile => $archive_path );
    $history->update;

    if ( $args{verbose} ) {
        STDOUT->print(
            qq{Archived $efshost:$path -> $archive_path\n},
        );
    }

    unlink( $path ) ||
        die qq{Unable to unlink $path: $ERRNO\n};

    return 1;

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
