#!/usr/bin/env perl

use strict;
use warnings;
use English;

use Test::More tests => 13;
use Test::Exception;
use List::MoreUtils qw(all);
use Try::Tiny;

use lib q{test/lib};
use EFS::Test::Config;

use blib;
use EFS;

## no critic: RequireExtendedFormatting

$ENV{EFS_RPC_DEBUG} = 1;

my $config = EFS::Test::Config->new;

my $name_local  = $config->get_value( q{LocalDevCell},  q{name} );
my $name_remote = $config->get_value( q{RemoteDevCell}, q{name} );

my $ldc_efsserver = $config->get_value( q{LocalDevCell}, q{efsserver} );

SKIP: {

    if ( not $config->get_value( q{Security}, q{kerberos_realm} ) ) {
        skip q{Kerberos tests not configured}, 1;
    }

    ## no critic: RequireInterpolationOfMetachars
    my $krb5ccname = qq{/var/tmp/krb5_efstest_$PID};
    $ENV{KRB5CCNAME} = qq{FILE:$krb5ccname};
    system qq{kinit -k -t /var/kerberos/keytabs/efsops.keytab efsops};
    ok( $CHILD_ERROR == 0, q{Acquired efsops credentials for this test} );

}

my $cell_rs = EFS::DBI->get_resultset(q{Cell});

my $cell_local  = $cell_rs->retrieve_bynames_assert( name => $name_local );
my $cell_remote = $cell_rs->retrieve_bynames_assert( name => $name_remote );

# There's only one per cell in the test setup
my ($efsserver) = $cell_local->efsservers();

throws_ok {
    EFS::RPC->execute(
        cell   => q{not a cell object},
        class  => q{EFS::RPC},
        method => q{test_success},
    );
}
qr{^The 'cell' parameter \("not a cell object"\) to}ms,
    q{testing cell argument, must be an object};

throws_ok {
    EFS::RPC->execute(
        cell   => $cell_local,
        class  => q{NOT::EFS},
        method => q{no_such_method},
    );
}
qr{Invalid argument:\n.*class => 'NOT::EFS'}ms,
    q{testing class argument, must match ^EFS::};

throws_ok {
    EFS::RPC->execute(
        cell   => $cell_local,
        class  => q{EFS::RPC},
        method => q{no_such_method},
    );
}
qr{Invalid arguments:\n.*class => 'EFS::RPC',\n.*method => 'no_such_method'}ms,
    q{testing method argument, must be a valid method};

throws_ok {
    EFS::RPC->execute(
        cell   => $cell_local,
        class  => q{EFS::RPC},
        method => q{test_success},
        args   => q{not an arrayref},
    );
}
qr{The 'args' parameter \("not an arrayref"\) to}ms,
    q{testing args argument, must be an array ref};

my $test_label = q{Testing local execution for real};
ok(
    EFS::RPC->execute(
        cell   => $cell_local,
        class  => q{EFS::RPC},
        method => q{test_success},
        args   => [ return => $test_label ],
    ) eq $test_label,
    $test_label
);

# OK.... NOW, we can start testing the actual code, through the daemon.

{
    my $success = 0;
    try {
        $success = EFS::RPC->execute(
            cell         => $cell_local,
            class        => q{EFS::RPC},
            method       => q{test_success},
            force_remote => 1,
        );
    }
    catch {
        warn qq{Test failed: $_\n};
    }
    finally {
        ok(
            $success,
            q{testing basic remote execution through the local cell (success)},
        );
    };
}

{
    my $eval_error = q{};
    try {
        EFS::RPC->execute(
            cell         => $cell_local,
            class        => q{EFS::RPC},
            method       => q{test_failure},
            args	     => [
                qq{test_failure, well... failed\n},
            ],
            force_remote => 1,
        );
    }
    catch {
        $eval_error = $_;
    }
    finally {
        ok(
            $eval_error =~ qr{test_failure, well... failed}ms,
            q{testing basic remote execution through the local cell (failure)},
        );
    };
}

my $firstline = q{this is the first line};
my $secondline = q{this is the second line};

{
    my $eval_error = q{};
    try {
        EFS::RPC->execute(
            cell         => $cell_local,
            class        => q{EFS::RPC},
            method       => q{test_failure},
            args	     => [ qq{$firstline\n$secondline\n} ],
            force_remote => 1,
        );
    }
    catch {
        $eval_error = $_;
    }
    finally {
        my @error = split m{\n+}x, $eval_error;

        ok( $error[0] =~ qr{$firstline}ms and $error[1] =~ qr{$secondline}ms,
            q{testing basic remote execution through the local cell (multiline error)} );
    };
}

{
    my $eval_error = q{};
    my $success = 0;
    try {
        $success = EFS::RPC->execute(
            cell   => $cell_remote,
            class  => q{EFS::RPC},
            method => q{test_success},
        );
    }
    catch {
        $eval_error = $_;
        warn qq{Test failed: $eval_error\n};
    }
    finally {
        ok(
            $success,
            q{testing basic remote execution through the remote cell (success)},
        );
    };
}

{
    my $eval_error = q{};
    try {
        EFS::RPC->execute(
            cell   => $cell_remote,
            class  => q{EFS::RPC},
            method => q{test_failure},
        );
    }
    catch {
        $eval_error = $_;
    }
    finally {
        ok(
            $eval_error,
            q{testing basic remote execution through the remote cell (failure)},
        );
    };
}

{
    my $eval_error = q{};
    my $success = 0;
    try {
        $success = EFS::RPC->execute(
            cell   => $cell_local,
            class  => q{EFS::RPC},
            method => q{test_success},
            args   => [
                return => {
                    scalar   => 1,
                    arrayref => [ 2 .. 4 ],
                    hashref  => { 5 => 6, 7 => 8, },
                },
            ],
            force_remote => 1,
        );
    }
    catch {
        $eval_error = $_;
        warn qq{Test failed: $eval_error\n};
    }
    finally {
        ok(
            all(
                sub { return $_ },
                ref $success eq q{HASH},
                $success->{scalar} == 1,
                ref $success->{arrayref} eq q{ARRAY},
                $success->{arrayref}->[0] == 2,
                $success->{arrayref}->[1] == 3,
                $success->{arrayref}->[2] == 4,
                ref $success->{hashref} eq q{HASH},
                $success->{hashref}->{5} == 6,
                $success->{hashref}->{7} == 8
            ),
            q{testing data structures in argument lists}
        );
    };
}

{
    my $eval_error = q{};
    my $return = undef;
    try {
        $return = EFS::RPC->execute(
            cell         => $cell_local,
            class        => q{EFS::RPC},
            method       => q{test_success},
            args         => [ return => 42, ],
            force_remote => 1,
        );
    }
    catch {
        warn qq{Test failed: $eval_error\n};
    }
    finally {
        ok( $return == 42, q{testing non-reference return value} );
    };
}

END {
    my $krb5ccname = qq{/var/tmp/krb5_efstest_$PID};
    if ( $krb5ccname && -f $krb5ccname ) {
        system q{kdestroy};
        $CHILD_ERROR and warn qq{Unable to destroy kerberos credentials\n};
    }
}
