#!/usr/bin/env perl

## no critic: RequireExtendedFormatting

use strict;
use warnings;
use English;

use File::Slurp;

use lib q{test/lib};
use EFS::Test;
use EFS::Test::Config;

use Test::More tests => 117;

my $test   = EFS::Test->new;
my $config = EFS::Test::Config->new;

my $upload_server   = $config->get_value( qw( Packaging upload_server ) );
my $upload_username = $config->get_value( qw( Packaging upload_username ) );
my $upload_sshkey   = $config->get_value( qw( Packaging upload_sshkey ) );
my $upload_root     = $config->get_value( qw( Packaging upload_root ) );

my $download_url    = $config->get_value( qw( Packaging download_url ) );
my ($download_dir)  = ( $download_url =~ m{^file://localhost(.*)}msx );

my @upload_projects   = qw( upload   upload-runtime   upload-build   upload-test );
my @download_projects = qw( download download-runtime download-build download-test );

purge_download_dirs(
    stage    => q{setup, phase one},
    projects => [ @upload_projects, @download_projects ],
);

my @setup = (
    q{efs create metaproj testpkgs efsops efsops},
);

my %attrs = (
    upload_server   => $upload_server,
    upload_username => $upload_username,
    upload_sshkey   => $upload_sshkey,
    upload_root     => $upload_root,
    download_url    => $download_url,
);

foreach my $key ( sort keys %attrs ) {
    push @setup, qq{efs setattr metaproj testpkgs $key $attrs{$key}};
}

foreach my $project ( @upload_projects ) {

    push @setup, (
        qq{efs create project testpkgs $project},
        qq{efs create release testpkgs $project testrel},
    );

    foreach my $install ( qw( common x86-64.rhel.5 x86-32.rhel.5 ) ) {
        push @setup, qq{efs create install testpkgs $project testrel $install};
        my $install_dir = qq{/efs/dev/testpkgs/$project/testrel/install/};
        $install_dir .= $install eq q{common} ? $install : qq{.exec/$install};
        push @setup, qq{touch $install_dir/notempty};
    }

    push @setup, (
        qq{efs checkpoint release testpkgs $project testrel},
    );

}

push @setup, (
    qq{efs define dependency testpkgs upload testrel testpkgs upload-runtime testrel runtime},
    qq{efs define dependency testpkgs upload testrel testpkgs upload-build testrel build},
    qq{efs define dependency testpkgs upload testrel testpkgs upload-test testrel test},
);

foreach my $command ( @setup ) {
    $test->efs_test_command(
        run_as  => q{efsops},
        name    => qq{(setup, phase one) $command},
        command => $command,
        bail    => 1,
    );
}

# We need to force the addition of the host key, so that subsequent
# commands have predictable output
$test->efs_test_command(
    run_as  => q{root},
    name    => q{(setup, phase one) Initializing ssh host key},
    command => qq{ssh -o StrictHostKeyChecking=no -i $upload_sshkey -l $upload_username $upload_server true},
    bail    => 1,
);

# End of setup

my @attrs = qw( runtime all );

foreach my $attr ( @attrs ) {

    if ( $attr ne q{runtime} ) {
        $test->efs_test_command(
            run_as  => q{efsops},
            command => qq{efs setattr metaproj testpkgs upload_depends $attr},
        );
    }

    $test->efs_test_command(
        run_as  => q{efsops},
        command => qq{efs upload release testpkgs upload testrel --force --recurse},
        expect  => [ upload_release_output( $attr ) ],
    );

}

# End of upload tests

convert_upload_to_download();

# Start of download tests

foreach my $attr ( @attrs ) {

    if ( $attr ne q{runtime} ) {
        $test->efs_test_command(
            run_as  => q{efsops},
            command => qq{efs setattr metaproj testpkgs download_depends $attr},
        );
        $test->efs_test_command(
            run_as  => q{efsops},
            command => qq{efs unlock release testpkgs download testrel},
        );
        $test->efs_test_command(
            run_as  => q{efsops},
            command => qq{efs unlock release testpkgs download-runtime testrel},
        );
    }

    $test->efs_test_command(
        run_as  => q{efsops},
        command => qq{efs download release testpkgs download testrel},
        filter  => qr{Downloading $download_url}ms,
        expect  => [ download_release_output( $attr ) ],
    );

}

# Start of teardown

my @teardown = ();

foreach my $type ( qw( upload download ) ) {
    push @teardown, (
        qq{efs define dependency testpkgs $type testrel testpkgs $type-runtime testrel runtime},
        qq{efs define dependency testpkgs $type testrel testpkgs $type-build testrel build},
        qq{efs define dependency testpkgs $type testrel testpkgs $type-test testrel test},
    );
}

foreach my $project ( @upload_projects, @download_projects ) {
    push @teardown, (
        qq{efs setstage release testpkgs $project testrel rejected},
        qq{efs destroy release testpkgs $project testrel},
        qq{efs purge release testpkgs $project testrel --now},
        qq{efs destroy project testpkgs $project},
    );
}

push @teardown, q{efs destroy metaproj testpkgs};

foreach my $command ( @teardown ) {
    $test->efs_test_command(
        run_as  => q{efsops},
        name    => qq{(teardown) $command},
        command => $command,
    );
}

purge_download_dirs(
    stage    => q{teardown},
    projects => [ @upload_projects, @download_projects ],
);

exit 0;

sub purge_download_dirs {

    my %args = @_;

    foreach my $project ( @{ $args{projects} } ) {
        my $dir = qq{$download_dir/testpkgs/$project};
        my $command = qq{su $upload_username -c 'rm -rf $dir'};
        $test->efs_test_command(
            run_as  => q{root},
            name    => qq{($args{stage}) $command},
            command => $command,
            bail    => 1,
        );
    }

    return 1;

}

sub upload_release_output {

    my $attr = shift;

    my @output = (
        package_release_output( q{upload-runtime} ),
    );

    if ( $attr eq q{all} ) {
        push @output, (
            package_release_output( q{upload-build} ),
            package_release_output( q{upload-test} ),
        );
    }

    push @output, (
        package_release_output( q{upload} ),
    );

    return @output;

}

sub package_release_output {

    my $project = shift;

    my $directory = qq{/efs/dev/testpkgs/$project/testrel/build/upload};

    return (
        qq{Validating the checkpoint state for release 'testpkgs/$project/testrel'.},
        qr{(Removed|No such) directory $directory}ms,
        qq{Created directory $directory},
        qq{Created directory $directory/package},
        qq{Created directory $directory/package/install},
        qq{Creating $directory/package/install/common.tar.bz2},
        qq{Creating $directory/package/install/x86-32.rhel.5.tar.bz2},
        qq{Creating $directory/package/install/x86-64.rhel.5.tar.bz2},
        qq{Packaged testpkgs/$project/testrel},
        qr{Uploaded testpkgs/$project/testrel package to}ms,
    );

}

sub download_release_output {

    my $attr = shift;

    my %depends = map { $_ => 1 } split m{,}msx, $attr;

    if ( $depends{all} ) {
        $depends{runtime}++;
        $depends{build}++;
        $depends{test}++;
    }

    my @output;

    if ( $depends{runtime} ) {
        push @output, (
            qq{Downloading $download_url/testpkgs/download-runtime/testrel/metadata.conf},
            qq{Downloading $download_url/testpkgs/download-runtime/testrel/install/common.tar.bz2},
            qq{Downloading $download_url/testpkgs/download-runtime/testrel/install/x86-32.rhel.5.tar.bz2},
            qq{Downloading $download_url/testpkgs/download-runtime/testrel/install/x86-64.rhel.5.tar.bz2},
        );
    }

    if ( $depends{all} ) {
        push @output, (
            qq{Downloading $download_url/testpkgs/download-build/testrel/metadata.conf},
            qq{Downloading $download_url/testpkgs/download-build/testrel/install/common.tar.bz2},
            qq{Downloading $download_url/testpkgs/download-build/testrel/install/x86-32.rhel.5.tar.bz2},
            qq{Downloading $download_url/testpkgs/download-build/testrel/install/x86-64.rhel.5.tar.bz2},
            qq{Downloading $download_url/testpkgs/download-test/testrel/metadata.conf},
            qq{Downloading $download_url/testpkgs/download-test/testrel/install/common.tar.bz2},
            qq{Downloading $download_url/testpkgs/download-test/testrel/install/x86-32.rhel.5.tar.bz2},
            qq{Downloading $download_url/testpkgs/download-test/testrel/install/x86-64.rhel.5.tar.bz2},
        );
    }

    push @output, (
        qq{Downloading $download_url/testpkgs/download/testrel/metadata.conf},
        qq{Downloading $download_url/testpkgs/download/testrel/install/common.tar.bz2},
        qq{Downloading $download_url/testpkgs/download/testrel/install/x86-32.rhel.5.tar.bz2},
        qq{Downloading $download_url/testpkgs/download/testrel/install/x86-64.rhel.5.tar.bz2},
    );

    return @output;

}

sub convert_upload_to_download {

    foreach my $upload ( @upload_projects ) {

        my $download = $upload;
        $download =~ s{upload}{download}gmsx;

        my $rsync = qq{rsync -az $download_dir/testpkgs/$upload/ $download_dir/testpkgs/$download};
        my $command = qq{su $upload_username -c '$rsync'};

        $test->efs_test_command(
            run_as  => q{root},
            name    => qq{(setup, phase two) $command},
            command => $command,
            bail    => 1,
        );

        my $file = qq{$download_dir/testpkgs/$download/testrel/metadata.conf};
        my $temp = qq{/var/tmp/testpkgs-$download-testrel-metadata.conf};

        if ( -f $temp ) {
            unlink $temp || die qq{Unable to unlink $temp: $ERRNO\n};
        }

        my $metadata = read_file( $file );
        $metadata =~ s{upload}{download}gmsx;
        write_file( $temp, $metadata );

        $test->efs_test_command(
            run_as  => q{root},
            name    => qq{(setup, phase two) edit $file},
            command => qq{su $upload_username -c 'cp $temp $file'},
        );

        unlink $temp || die qq{Unable to unlink $temp: $ERRNO\n};

    }

}
