#!/usr/bin/env perl

use strict;
use warnings;
use English;

use Test::More tests => 51;
use Try::Tiny;

use blib;
use EFS;

use lib q{test/lib};
use EFS::Test;

my $test = EFS::Test->new;
my $config = $test->get_config;

my $username  = $test->get_primary_username;
my $groupname = $test->get_primary_groupname;

my $ldc_name      = $config->get_value( q{LocalDevCell}, q{name} );
my $ldc_shortname = $config->get_value( q{LocalDevCell}, q{shortname} );
my $ldc_efsserver = $config->get_value( q{LocalDevCell}, q{efsserver} );

my $rdc_name      = $config->get_value( q{RemoteDevCell}, q{name} );
my $rdc_shortname = $config->get_value( q{RemoteDevCell}, q{shortname} );
my $rdc_efsserver = $config->get_value( q{RemoteDevCell}, q{efsserver} );

my $lpc_name      = $config->get_value( q{LocalProdCell}, q{name} );

my $root_domain = $config->get_value( q{Global}, q{root_domain} );
my $ownerlist = join q{@}, $username, $root_domain;

my $fileqtree_rs = EFS::DBI->get_resultset( q{Fileqtree} );

my $ldc_qtree = $fileqtree_rs->retrieve_bynames_assert(
    cell       => $ldc_name,
    fileserver => $config->get_value( q{LocalDevCell}, q{fileserver} ),
    fileshare  => $config->get_value( q{LocalDevCell}, q{distshare} ),
    name       => q{efs_qtree},
);
my $local_admin_path = $ldc_qtree->admin_path;

my $rdc_qtree = $fileqtree_rs->retrieve_bynames_assert(
    cell       => $rdc_name,
    fileserver => $config->get_value( q{RemoteDevCell}, q{fileserver} ),
    fileshare  => $config->get_value( q{RemoteDevCell}, q{distshare} ),
    name       => q{efs_qtree},
);
my $remote_admin_path = $rdc_qtree->admin_path;

my $cell_rs = EFS::DBI->get_resultset( q{Cell} );
if ( my @hidden = $cell_rs->search({ hidden => 1 }) ) {
    $test->efs_test_command(
        name    => q{Hidden cells: } . join( q{,}, map { $_->name } @hidden ),
        command => q{false},
        bail    => 1,
    );
}

my $project_name = q{testclone clonecellstate};

my @setup = (
    qq{efs create metaproj testclone $username $groupname $ownerlist},
    qq{efs create project $project_name $ldc_name},
);

foreach my $release ( qw( local remote ) ) {
    push @setup, (
        qq{efs create release $project_name $release},
        qq{efs create install $project_name $release common},
        qq{touch /efs/dev/testclone/clonecellstate/$release/install/common/foo},
        qq{efs checkpoint release $project_name $release},
        qq{efs create releasealias $project_name $release-rl $release},
    );
}

push @setup, (
    qq{efs dist release $project_name local -cells $ldc_name},
    qq{efs dist releasealias $project_name local-rl -cells $ldc_name},
    qq{efs dist release $project_name remote -cells $rdc_name},
    qq{efs dist releasealias $project_name remote-rl -cells $rdc_name},
);

foreach my $setup (@setup) {
    my $run_as = ( $setup =~ m{^ touch \s+}msx ) ? q{primary_user} : q{efsops};
    $test->efs_test_command(
        run_as  => $run_as,
        name    => qq{(setup) $setup},
        command => $setup,
        bail    => 1,
    );
}

$test->efs_test_command(
    command => qq{efs diststate release $project_name local},
    filter  => qr{ ^ ( Install | common ) \s+ }msx,
    expect  => [
        qr{ Install \s+ $ldc_shortname \s+ $rdc_shortname }msx,
        qr{ common \s+ D \s+ X }msx,
    ],
);

$test->efs_test_command(
    command => qq{efs diststate release $project_name remote},
    filter  => qr{ ^ ( Install | common ) \s+ }msx,
    expect  => [
        qr{ Install \s+ $ldc_shortname \s+ $rdc_shortname }msx,
        qr{ common \s+ X \s+ D }msx,
    ],
);

$test->efs_test_command(
    command => qq{efs diststate releasealias $project_name local-rl},
    filter  => qr{ ^ ( Cell | $rdc_shortname | $ldc_shortname ) \s+ }msx,
    expect  => [
        qr{ Cell \s+ Target }msx,
        qr{ $rdc_shortname \s+ --- }msx,
    ],
);

$test->efs_test_command(
    command => qq{efs diststate releasealias $project_name remote-rl},
    filter  => qr{ ^ ( Cell | $rdc_shortname | $ldc_shortname ) \s+ }msx,
    expect  => [
        qr{ Cell \s+ Target }msx,
        qr{ $ldc_shortname \s+ --- }msx,
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs clonecellstate cell $ldc_name $rdc_name},
    not     => 1,
    expect  =>
        q{This operation can only be run when EFS is in maintenance mode},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => q{efs define parameter maintenance_mode true},
);

my $history_rs = EFS::DBI->get_resultset( q{History} );
my $history = $history_rs->create({
    command    => q{dist release},
    request    => q{dist release any old thing},
    authuser   => q{efsops},
    efshost    => $ldc_efsserver,
    efsversion => q{test},
    pid        => $PID,
});

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs clonecellstate cell $ldc_name $rdc_name},
    not     => 1,
    expect  => [
        q{The following dist commands are still active:},
        qr{Id \s+ Authuser \s+ Efshost \s+ Pid \s+ Request }msx,
        qr{ ^ =+ }msx,
        qr{ \d+ \s+ efsops \s+ $ldc_efsserver \s+ $PID \s+
            dist\srelease\sany\sold\sthing }msx,
        q{This command may not be used while active dists are in progress},
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs clonecellstate cell $ldc_name $ldc_name --force},
    not     => 1,
    expect  => q{Cell names may not match},
);

$history->delete;

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs clonecellstate cell $ldc_name $ldc_shortname},
    not     => 1,
    expect  => q{Cell names may not match},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs clonecellstate cell $ldc_name $lpc_name},
    not     => 1,
    expect  => q{Cell types must match},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs hide cell $lpc_name},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs hide cell $ldc_name},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs clonecellstate cell $ldc_name $rdc_name},
    not     => 1,
    expect  => q{From cell must NOT be hidden},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs expose cell $ldc_name},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs expose cell $lpc_name},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs clonecellstate cell $ldc_name $rdc_name},
    not     => 1,
    expect  => q{To cell must be hidden},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs hide cell $rdc_name},
);

{

    my $command = q{true};

    try {
        EFS::Rsync->new({
            dest_efsserver_name => $rdc_efsserver,
            src_path            => $local_admin_path,
            dest_path           => $remote_admin_path,
        });
    }
    catch {
        $command = q{false};
    };

    $test->efs_test_command(
        name    => q{EFS::Rsync local -> remote},
        command => $command,
    );

}

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs clonecellstate cell $ldc_name $rdc_name},
    expect  => [
        qr{Deleting \s+ \d+ \s+ install \s+ records \s+ for \s+ $rdc_name }msx,
        qr{Cloning \s+ \d+ \s+ install \s+ records \s+ for \s+ $rdc_name }msx,
        qr{Deleting \s+ \d+ \s+ releasealias \s+ records \s+ for \s+ $rdc_name }msx,
        qr{Cloning \s+ \d+ \s+ releasealias \s+ records \s+ for \s+ $rdc_name }msx,
        qq{Cloned cell state from $ldc_name to $rdc_name},
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs expose cell $rdc_name},
);

$test->efs_test_command(
    command => qq{efs diststate release $project_name local},
    filter  => qr{ up \s+ to \s+ date }msx,
    expect  => q{Dev environment is up to date.},
);

$test->efs_test_command(
    command => qq{efs diststate release $project_name remote},
    filter  => qr{ ^ ( Install | common ) \s+ }msx,
    expect  => [
        qr{ Install \s+ $ldc_shortname \s+ $rdc_shortname }msx,
        qr{ common \s+ X \s+ X }msx,
    ],
);

$test->efs_test_command(
    command => qq{efs diststate releasealias $project_name local-rl},
    filter  => qr{ up \s+ to \s+ date }msx,
    expect  => q{Dev environment is up to date.},
);

$test->efs_test_command(
    command => qq{efs diststate releasealias $project_name remote-rl},
    filter  => qr{ ^ ( Cell | $rdc_shortname | $ldc_shortname ) \s+ }msx,
    expect  => [
        qr{ Cell \s+ Target }msx,
        qr{ $ldc_shortname \s+ --- }msx,
        qr{ $rdc_shortname \s+ --- }msx,
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => q{efs undef parameter maintenance_mode},
);

my @teardown = ();

foreach my $release ( qw( local remote ) ) {
    push @teardown, (
        qq{efs destroy releasealias $project_name $release-rl},
        qq{efs setstage release $project_name $release rejected},
        qq{efs destroy release $project_name $release},
        qq{efs purge release $project_name $release --now},
    );
}

push @teardown, (
    qq{efs destroy project $project_name},
    q{efs destroy metaproj testclone},
);

foreach my $teardown ( @teardown ) {
    $test->efs_test_command(
        run_as  => q{efsops},
        name    => qq{(teardown) $teardown},
        command => $teardown,
    );
}

