package EFS::Test::Config;

use strict;
use warnings;
use English;

use Moose;
use Params::Validate qw(:all);
use Config::Tiny;
use File::Basename qw( dirname );

has [qw( file data )] => (
    is => 'rw',
);

{

    sub BUILD {

        my ( $self, $args_ref ) = @_;

        my @args = %{$args_ref};

        my (%args) = validate( @args, {
            noexpand => { type => SCALAR, optional => 1 }
        });

        my $file = $ENV{EFS_TEST_CONFIG} || q{/etc/efs/test.conf};

        $self->file( $file );

        if ( -f $file ) {
            my $conf_data = Config::Tiny->read( $file ) ||
                die qq{Unable to open $file: $ERRNO\n};
            $self->data( $conf_data );
        }
        else {
            $self->data( Config::Tiny->new );
        }

        if ( $args{noexpand} ) {
            return;
        }

        my $data = $self->data;

        $data->{Global}->{root_domain}     ||= q{test.efs};
        $data->{Global}->{primary_user}    ||= q{efstest};
        $data->{Global}->{secondary_user}  ||= q{efsalter};
        $data->{Global}->{secondary_group} ||= q{efsalter};
        $data->{Global}->{sshcmd}          ||=
            q{ssh -o LogLevel=ERROR -o StrictHostKeyChecking=no};

        my $root_domain = $data->{Global}->{root_domain};

        $data->{LocalDevCell}->{efsserver}  ||= qq{efsserver1.$root_domain};
        $data->{RemoteDevCell}->{efsserver} ||= qq{efsserver2.$root_domain};

        $data->{LocalProdCell}->{efsserver} =
            $data->{LocalDevCell}->{efsserver};

        $data->{Filesystem}->{type} ||= q{nfs};

        my $secondary_user = $data->{Global}->{secondary_user};

        if ( not defined scalar getpwnam( $secondary_user ) ) {
            die( <<"INVALID_SECONDARY_USER" );
An invalid user name has been specified for use in the test environment.
Please provide a valid username for secondary_user in $file.
INVALID_SECONDARY_USER
        }

        my $secondary_group = $data->{Global}->{secondary_group};

        if ( not defined scalar getgrnam( $secondary_group ) ) {
            die( <<"INVALID_SECONDARY_GROUP" );
An invalid group name has been specified for use in the test environment.
Please provide a valid groupname for secondary_group in $file.
INVALID_SECONDARY_GROUP
        }

        foreach my $key (qw( region campus location )) {
            $data->{Global}->{$key} ||= substr( $key, 0, 3 );
        }

        my $region   = $data->{Global}->{region};
        my $campus   = $data->{Global}->{campus};
        my $location = $data->{Global}->{location};

        $data->{Database}->{port} ||= 17_833;
        $data->{Database}->{user} ||= q{efsops};

        if ( $data->{Database}->{connect} ) {
            my ($dbvendor) = ( split m{:}msx, $data->{Database}->{connect} )[1];
            $data->{Database}->{dbvendor} = lc($dbvendor);
        }

        my %celltag_of = (
            LocalDevCell  => q{a},
            RemoteDevCell => q{b},
            LocalProdCell => q{p},
        );

        foreach my $section (qw( LocalDevCell RemoteDevCell LocalProdCell )) {

            $data->{$section}->{name} =
                join ( q{.},
                       $celltag_of{$section}, $location, $campus,
                       $region, $root_domain );

            my $celltag = $celltag_of{$section};

            $data->{$section}->{name} = qq{$celltag.$location.$campus.$region.$root_domain};
            $data->{$section}->{shortname} = qq{$celltag.$location};

            if ( $data->{Filesystem}->{type} eq q{afs} ) {

                my $fileserver =
                    $celltag eq q{a} ?
                        qq{${location}core.$root_domain} :
                            qq{${location}afs${celltag}01.$root_domain};

                $data->{$section}->{fileserver} = $fileserver;
                $data->{$section}->{afsshares}  = qq{/vicepa /vicepb};

                if ( $section ne q{LocalProdCell} ) {
                    # XXX: This 02 host isn't being built by efs_afs_setup yet
                    # $data->{$section}->{fileserver} .= qq{ ${location}afs${celltag}02.$root_domain};
                }

                
            } else {

                if ( $data->{$section}->{fileserver} ) {
                    $data->{$section}->{devshare}  = qq{/vol/efs_dev_${celltag}_$location};
                    $data->{$section}->{homeshare} = qq{/vol/efs_home_${celltag}_$location};
                    $data->{$section}->{distshare} = q{/vol/efs_dist};
                } else {
                    if ( $section ne q{LocalProdCell} ) {
                        $data->{$section}->{fileserver} = $data->{$section}->{efsserver};
                    } else {
                        my $name = $data->{$section}->{efsserver};
                        $name =~ s/^[^\.]+\./fakeserver./msx;
                        $data->{$section}->{fileserver} = $name;
                    }
                    $data->{$section}->{devshare}  = qq{/var/tmp/efs_dev_${celltag}_$location};
                    $data->{$section}->{homeshare} = qq{/var/tmp/efs_home_${celltag}_$location};
                    $data->{$section}->{distshare} = qq{/var/tmp/efs_dist_${celltag}_$location};
                }

            }

        }

        return;

    }

    sub delete_key {

        my ($self, $section, $key) = @_;

        delete $self->data->{$section}->{$key};

        return 1;

    }

    sub get_value {

        my ($self, $section, $key) = @_;

        return $self->data->{$section}->{$key};

    }

    sub set_value {

        my ($self, $section, $key, $value) = @_;

        return $self->data->{$section}->{$key} = $value;

    }

    sub save_file {

        my $self  = shift;

        my $file = $self->file;

        my $dir  = dirname $file;

        if ( not -d $dir ) {
            mkdir( $dir, 0755 ) ||
                die qq{Unable to mkdir $dir: $ERRNO\n};
        }

        if ( not -f $file ) {
            system( qq{touch $file } );
            if ( $CHILD_ERROR ) {
                die qq{Unable to touch $file\n};
            }
        }

        return $self->data->write( $self->file );

    }

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
